% Program description
% This function generates a vector of expected casflows for a given CDO
% contract using Monte Carlo simulation. It is assumed that all obligors in
% the contract have the same default rate "default_rate" and uncorrelated.
% The component i of the vector represents the expected cashflow of the
% tranche i.

function cash_flows = CDO_cashflow_generator(attachment_points, tranche_prices, equity_tranche_up_front_payment,...
                                         contract_period, hazard_rate, interest_rate)

%  The possible cashflows are simulated "number_simulations_=" 10 000 times
%  to get the average.
number_of_simulations=10000;
% Number of obligors (for CDX contract is assumed to be 125)
number_of_obligors = 125;
% Number of payments per year (for CDX contract is assumed to be 4 - quaterly payments)
payment_frequency = 4;

recovery_rate=0.4;
%periodical payments
tranche_prices_periodic=zeros(1,length(attachment_points)-1);
for i=1:length(attachment_points)-1
  tranche_prices_periodic(i) = tranche_prices(i)/payment_frequency; 
end

% Matrix of defaults (1 column - 1 scenario)
%default_times_matrix = exprnd(-contract_period/log(1-default_rate), number_of_obligors, number_of_simulations); 

%calculate cashflow for every simulation
cash_flows = zeros(length(attachment_points)-1,1);
for step=1:number_of_simulations
   default_times_vector=exprnd(1/hazard_rate, number_of_obligors, 1);
   %vector of tranche sizes
   tranche_size=zeros(1,length(attachment_points)-1);
   for i=1:length(attachment_points)-1
       tranche_size(i) = attachment_points(i+1)-attachment_points(i);
   end    
   
   %defaults counter vector
   default_counter=zeros(payment_frequency*contract_period,1);  
      for i=1:number_of_obligors
          if(default_times_vector(i)<contract_period)
            k=0;
            if(int32(default_times_vector(i)*payment_frequency)<default_times_vector(i)*payment_frequency)
               k=1;
            end   
            default_counter(int32(default_times_vector(i)*payment_frequency)+k)= default_counter(int32(default_times_vector(i)*payment_frequency)+k)+1;
          end      
      end
   % cumulative loss (percentage)   
   cumulative_loss(1) = default_counter(1)*(1-recovery_rate)/number_of_obligors;
     for i=2:payment_frequency*contract_period
         cumulative_loss(i)=cumulative_loss(i-1)+default_counter(i)*(1-recovery_rate)/number_of_obligors;
     end    
    number_of_tranches = length(attachment_points)-1;
   
   % matrix of tranche losses
    for n=1:number_of_tranches 
      for j=1:payment_frequency*contract_period
        total_tranche_losses(n,j)=min(attachment_points(n+1)-attachment_points(n),max(cumulative_loss(j)-attachment_points(n),0));
      end
    end
    
    periodical_tranche_losses(:,1)=total_tranche_losses(:,1);
    for j=2:payment_frequency*contract_period
      periodical_tranche_losses(:,j)=total_tranche_losses(:,j)-total_tranche_losses(:,j-1);
    end
  
   %matrix of tranche gains
     for n=1:number_of_tranches 
      for j=1:payment_frequency*contract_period
        periodical_tranche_gains(n,j)=tranche_prices_periodic(n)*(tranche_size(n)-total_tranche_losses(n,j));
      end
    end    
    

   % calculate cashflow of tranches for every period 
   current_cash_flow = zeros(length(attachment_points)-1,1);%initial cashflow   
   current_cash_flow(1)=equity_tranche_up_front_payment*(attachment_points(2)-attachment_points(1));      
      for i=1:payment_frequency*contract_period
        discount_factor=1/(1+interest_rate/payment_frequency)^i; 
        current_cash_flow=current_cash_flow+discount_factor*(periodical_tranche_gains(:,i)-periodical_tranche_losses(:,i));
      end

  cash_flows = cash_flows+current_cash_flow;
end

cash_flows=cash_flows/number_of_simulations;


clear number_of_simulations number_of_obligors default_times_matrix default_times_vector;
clear step payment_frequency i j n discount_factor defaults_percentage;
clear current_cash_flow tranche_size cumulative_loss total_tranche_losses periodical_tranche_losses periodical_tranche_gains;
clear discount_factor;

end